@@ -5,19 +5,18 @@ RDF triple store classes.
 
 Copyright (C) 2006, 2007 Paul Boddie <paul@boddie.org.uk>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) any later version.
+This program is free software; you can redistribute it and/or modify it under
+the terms of the GNU Lesser General Public License as published by the Free
+Software Foundation; either version 3 of the License, or (at your option) any
+later version.
 
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
+This program is distributed in the hope that it will be useful, but WITHOUT
+ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
+details.
 
-You should have received a copy of the GNU Lesser General Public
-License along with this library; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
+You should have received a copy of the GNU Lesser General Public License along
+with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
 from sqltriples.Types import *
@@ -44,7 +43,8 @@ class TripleStore:
     codes_to_names = {
         "U" : "URIRef",
         "L" : "Literal",
-        "B" : "BNode"
+        "B" : "BNode",
+        None : "URIRef" # supports predicate queries
         }
 
     # We need a non-null name for "null" contexts.
@@ -84,6 +84,8 @@ class TripleStore:
         # Useful classes.
 
         self.Expression = Expression
+        self.LiteralExpression = LiteralExpression
+        self.URIRefExpression = URIRefExpression
         self.Pattern = Pattern
         self.Defined = Defined
 
@@ -173,7 +175,7 @@ class TripleStore:
         Instantiate and return an object from the given 'value' and 'type_code'.
         """
 
-        return globals()[self.codes_to_names.get(type_code, "Literal")](value)
+        return getattr(self, self.codes_to_names.get(type_code, "Literal"))(value)
 
     def _get_bnode_id(self):
 
@@ -211,6 +213,14 @@ class TripleStore:
         else:
             return "%s_%s" % (self.table_name, index)
 
+    def _findall(self, s, c):
+        l = []
+        i = s.find(c)
+        while i != -1:
+            l.append(i)
+            i = s.find(c, i+1)
+        return l
+
     def _get_condition(self, value, column_name, conditions, values, index, new_index):
 
         """
@@ -233,13 +243,35 @@ class TripleStore:
 
         # Process an expression.
 
-        elif isinstance(value, Expression) or isinstance(value, tuple):
+        elif isinstance(value, self.Expression) or isinstance(value, tuple):
 
             # Add the expression as a condition in the generated query.
 
             comparison, comparison_values = value
+
+            # Add the values, processing sequences specially.
+
+            n = 0
+            for comparison_value in comparison_values:
+
+                # For sequences given as values, make the parameter more
+                # complicated, transforming a single placeholder into a list of
+                # placeholders of the appropriate length.
+
+                if isinstance(comparison_value, tuple) or isinstance(comparison_value, list):
+
+                    parameter_location = self._findall(comparison, "?")[n]
+                    comparison = comparison[:parameter_location] + "(" + \
+                        ", ".join(["?"] * len(comparison_value)) + \
+                        ")" + comparison[parameter_location + 1:]
+                    values += tuple(map(unicode, comparison_value))
+
+                else:
+                    values.append(unicode(comparison_value))
+
+                n += 1
+
             conditions.append(comparison.replace("_", self._column(column_name, index)))
-            values += map(unicode, comparison_values)
 
             if column_name != "predicate":
                 conditions.append("%s = ?" % self._column(column_name + "_type", index))
@@ -277,12 +309,16 @@ class TripleStore:
             # must match the stated column's value.
 
             if value:
-                conditions.append("%s in (%s)" % (
-                    self._column(column_name, index),
-                    ", ".join(["?"] * len(value))
-                    ))
-                for v in value:
-                    values.append(unicode(v))
+                if len(value) == 1:
+                    conditions.append("%s = ?" % self._column(column_name, index))
+                    values.append(unicode(value[0]))
+                else:
+                    conditions.append("%s in (%s)" % (
+                        self._column(column_name, index),
+                        ", ".join(["?"] * len(value))
+                        ))
+                    for v in value:
+                        values.append(unicode(v))
             else:
                 conditions.append("%s <> %s" % (
                     self._column(column_name, index),
